@@ -13,6 +13,7 @@ from lutris.gui.widgets.common import EditableGrid, FileChooserEntry, Label, VBo
 from lutris.gui.widgets.searchable_combobox import SearchableCombobox
 from lutris.runners import InvalidRunner, import_runner
 from lutris.util.log import logger
+from lutris.util.strings import gtk_safe
 
 
 class ConfigBox(VBox):
@@ -34,6 +35,62 @@ class ConfigBox(VBox):
         self.reset_buttons = {}
         self.wrappers = {}
         self.warning_boxes = {}
+        self.error_boxes = {}
+        self._advanced_visibility = False
+        self._filter = ""
+
+    @property
+    def advanced_visibility(self):
+        return self._advanced_visibility
+
+    @advanced_visibility.setter
+    def advanced_visibility(self, value):
+        """Sets the visibility of every 'advanced' option and every section that
+        contains only 'advanced' options."""
+        self._advanced_visibility = value
+        self.update_option_visibility()
+
+    @property
+    def filter(self):
+        return self._filter
+
+    @filter.setter
+    def filter(self, value):
+        """Sets the visibility of the options that have some text in the label or
+        help-text."""
+        self._filter = value
+        self.update_option_visibility()
+
+    def update_option_visibility(self):
+        """Recursively searches out all the options and shows or hides them according to
+        the filter and advanced-visibility settings."""
+
+        def update_widgets(widgets):
+            filter_text = self.filter.lower()
+
+            visible_count = 0
+            for widget in widgets:
+                if isinstance(widget, ConfigBox.SectionFrame):
+                    frame_visible_count = update_widgets(widget.vbox.get_children())
+                    visible_count += frame_visible_count
+                    widget.set_visible(frame_visible_count > 0)
+                    widget.set_frame_visible(frame_visible_count > 1)
+                else:
+                    widget_visible = self.advanced_visibility or not widget.get_style_context().has_class("advanced")
+                    if widget_visible and filter_text and hasattr(widget, "lutris_option_label"):
+                        label = widget.lutris_option_label.lower()
+                        helptext = widget.lutris_option_helptext.lower()
+                        if filter_text not in label and filter_text not in helptext:
+                            widget_visible = False
+                    widget.set_visible(widget_visible)
+                    widget.set_no_show_all(not widget_visible)
+                    if widget_visible:
+                        visible_count += 1
+                        widget.show_all()
+
+            return visible_count
+
+        update_widgets(self.get_children())
 
     def generate_top_info_box(self, text):
         """Add a top section with general help text for the current tab"""
@@ -165,77 +222,44 @@ class ConfigBox(VBox):
                 hbox.pack_start(self.wrapper, True, True, 0)
 
                 if "warning" in option:
+                    option_body = option_container
                     option_container = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, visible=True)
-                    option_container.pack_start(hbox, False, False, 0)
-                    warning = ConfigBox.WarningBox(option["warning"])
-                    warning.set_margin_left(18)
-                    warning.set_margin_right(18)
-                    warning.set_margin_bottom(6)
+                    option_container.pack_start(option_body, False, False, 0)
+                    warning = ConfigWarningBox(option["warning"], option_key)
                     warning.update_warning(self.config)
                     self.warning_boxes[option_key] = warning
                     option_container.pack_start(warning, False, False, 0)
 
+                if "error" in option:
+                    option_body = option_container
+                    option_container = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, visible=True)
+                    option_container.pack_start(option_body, False, False, 0)
+                    error = ConfigErrorBox(option["error"], option_key, hbox)
+                    error.update_warning(self.config)
+                    self.error_boxes[option_key] = error
+                    option_container.pack_start(error, False, False, 0)
+
                 # Hide if advanced
                 if option.get("advanced"):
                     option_container.get_style_context().add_class("advanced")
 
+                option_container.lutris_option_key = option_key
+                option_container.lutris_option_label = option["label"]
+                option_container.lutris_option_helptext = option.get("help") or ""
                 current_vbox.pack_start(option_container, False, False, 0)
             except Exception as ex:
                 logger.exception("Failed to generate option widget for '%s': %s", option.get("option"), ex)
         self.show_all()
 
         show_advanced = settings.read_setting("show_advanced_options") == "True"
-        self.set_advanced_visibility(show_advanced)
+        self.advanced_visibility = show_advanced
 
     def update_warnings(self):
         for box in self.warning_boxes.values():
             box.update_warning(self.config)
 
-    class WarningBox(Gtk.Box):
-        def __init__(self, warning):
-            self.warning = warning
-            super().__init__(spacing=6, visible=False, no_show_all=True)
-            warning_image = Gtk.Image(visible=True)
-            warning_image.set_from_icon_name("dialog-warning", Gtk.IconSize.DND)
-            self.pack_start(warning_image, False, False, 0)
-            self.warning_label = Gtk.Label(visible=True, xalign=0)
-            self.warning_label.set_line_wrap(True)
-            self.pack_start(self.warning_label, False, False, 0)
-
-        def update_warning(self, config):
-            if callable(self.warning):
-                text = self.warning(config)
-            else:
-                text = self.warning
-
-            if text:
-                self.warning_label.set_markup(str(text))
-
-            self.set_visible(bool(text))
-
-    def set_advanced_visibility(self, value):
-        """Sets the visibility of every 'advanced' option and every section that
-        contains only 'advanced' options."""
-
-        def update_widgets(widgets):
-            visible_count = 0
-            for widget in widgets:
-                if isinstance(widget, ConfigBox.SectionFrame):
-                    frame_visible_count = update_widgets(widget.vbox.get_children())
-                    visible_count += frame_visible_count
-                    widget.set_visible(frame_visible_count > 0)
-                    widget.set_frame_visible(frame_visible_count > 1)
-                else:
-                    widget_visible = value or not widget.get_style_context().has_class("advanced")
-                    widget.set_visible(widget_visible)
-                    widget.set_no_show_all(not widget_visible)
-                    if widget_visible:
-                        visible_count += 1
-                        widget.show_all()
-
-            return visible_count
-
-        update_widgets(self.get_children())
+        for box in self.error_boxes.values():
+            box.update_warning(self.config)
 
     def call_widget_generator(self, option, option_key, value, default):  # noqa: C901
         """Call the right generation method depending on option type."""
@@ -282,6 +306,8 @@ class ConfigBox(VBox):
             self.generate_directory_chooser(option, value)
         elif option_type == "file":
             self.generate_file_chooser(option, value)
+        elif option_type == "command_line":
+            self.generate_file_chooser(option, value, shell_quoting=True)
         elif option_type == "multiple":
             self.generate_multiple_file_chooser(option_key, option["label"], value)
         elif option_type == "label":
@@ -431,7 +457,7 @@ class ConfigBox(VBox):
         self.option_changed(spin_button, option, value)
 
     # File chooser
-    def generate_file_chooser(self, option, path=None):
+    def generate_file_chooser(self, option, text=None, shell_quoting=False):
         """Generate a file chooser button to select a file."""
         option_name = option["option"]
         label = Label(option["label"])
@@ -439,8 +465,9 @@ class ConfigBox(VBox):
         file_chooser = FileChooserEntry(
             title=_("Select file"),
             action=Gtk.FileChooserAction.OPEN,
-            path=path,
-            default_path=default_path
+            text=text,
+            default_path=default_path,
+            shell_quoting=shell_quoting
         )
         # file_chooser.set_size_request(200, 30)
 
@@ -449,19 +476,20 @@ class ConfigBox(VBox):
             if default_path and os.path.exists(default_path):
                 file_chooser.entry.set_text(default_path)
 
-        if path:
+        if text:
             # If path is relative, complete with game dir
-            if not os.path.isabs(path):
-                path = os.path.expanduser(path)
-                if not os.path.isabs(path):
+            if not os.path.isabs(text):
+                text = os.path.expanduser(text)
+                if not os.path.isabs(text):
                     if self.game and self.game.directory:
-                        path = os.path.join(self.game.directory, path)
-            file_chooser.entry.set_text(path)
+                        text = os.path.join(self.game.directory, text)
+            file_chooser.entry.set_text(text)
 
         file_chooser.set_valign(Gtk.Align.CENTER)
         self.wrapper.pack_start(label, False, False, 0)
         self.wrapper.pack_start(file_chooser, True, True, 0)
         self.option_widget = file_chooser
+
         file_chooser.connect("changed", self._on_chooser_file_set, option_name)
 
     # Directory chooser
@@ -473,7 +501,7 @@ class ConfigBox(VBox):
         if not path and self.game and self.game.runner:
             default_path = self.game.runner.working_dir
         directory_chooser = FileChooserEntry(
-            title=_("Select folder"), action=Gtk.FileChooserAction.SELECT_FOLDER, path=path, default_path=default_path
+            title=_("Select folder"), action=Gtk.FileChooserAction.SELECT_FOLDER, text=path, default_path=default_path
         )
         directory_chooser.connect("changed", self._on_chooser_file_set, option_name)
         directory_chooser.set_valign(Gtk.Align.CENTER)
@@ -733,3 +761,53 @@ class SystemBox(ConfigBox):
                 "If modified, these options supersede the same options from "
                 "the global preferences."
             ))
+
+
+class ConfigMessageBox(Gtk.Box):
+    def __init__(self, warning, option_key, icon_name):
+        self.warning = warning
+        self.option_key = option_key
+        super().__init__(spacing=6, visible=False, no_show_all=True)
+        self.set_margin_left(18)
+        self.set_margin_right(18)
+        self.set_margin_bottom(6)
+
+        image = Gtk.Image(visible=True)
+        image.set_from_icon_name(icon_name, Gtk.IconSize.DND)
+        self.pack_start(image, False, False, 0)
+        self.label = Gtk.Label(visible=True, xalign=0)
+        self.label.set_line_wrap(True)
+        self.pack_start(self.label, False, False, 0)
+
+    def update_warning(self, config):
+        try:
+            if callable(self.warning):
+                text = self.warning(config, self.option_key)
+            else:
+                text = self.warning
+        except Exception as err:
+            logger.exception("Unable to generate configuration warning: %s", err)
+            text = gtk_safe(err)
+
+        if text:
+            self.label.set_markup(str(text))
+
+        visible = bool(text)
+        self.set_visible(visible)
+        return visible
+
+
+class ConfigWarningBox(ConfigMessageBox):
+    def __init__(self, warning, option_key):
+        super().__init__(warning, option_key, icon_name="dialog-warning")
+
+
+class ConfigErrorBox(ConfigMessageBox):
+    def __init__(self, error, option_key, option_container):
+        super().__init__(error, option_key, icon_name="dialog-error")
+        self.option_container = option_container
+
+    def update_warning(self, config):
+        visible = super().update_warning(config)
+        self.option_container.set_sensitive(not visible)
+        return visible
