@@ -19,7 +19,7 @@ _supported_scale_factors = {
 }
 
 
-def _get_opengl_warning(config):
+def _get_opengl_warning(config, _option_key):
     if "scaler" in config and "renderer" in config:
         renderer = config["renderer"]
         if renderer and renderer != "software":
@@ -30,7 +30,7 @@ def _get_opengl_warning(config):
     return None
 
 
-def _get_scale_factor_warning(config):
+def _get_scale_factor_warning(config, _option_key):
     """Generate a warning message for when the scaler and scale-factor can't be used together."""
     if "scaler" in config and "scale-factor" in config:
         scaler = config["scaler"]
@@ -49,6 +49,7 @@ class scummvm(Runner):
     platforms = [_("Linux")]
     runnable_alone = True
     runner_executable = "scummvm/bin/scummvm"
+    flatpak_id = "org.scummvm.ScummVM"
     game_options = [
         {
             "option": "game_id",
@@ -471,14 +472,21 @@ class scummvm(Runner):
     def game_path(self):
         return self.game_config.get("path")
 
-    @property
-    def libs_dir(self):
+    def get_extra_libs(self):
+        """Scummvm runner ships additional, they may be removed in a future version."""
+        base_runner_path = os.path.join(settings.RUNNER_DIR, "scummvm")
+        if not self.get_executable().startswith(base_runner_path):
+            return ""
         path = os.path.join(settings.RUNNER_DIR, "scummvm/lib")
         return path if system.path_exists(path) else ""
 
     def get_command(self):
-        return [
-            self.get_executable(),
+        command = super().get_command()
+        if not command:
+            return []
+        if "flatpak" in command[0]:
+            return command
+        return command + [
             "--extrapath=%s" % self.get_scummvm_data_dir(),
             "--themepath=%s" % self.get_scummvm_data_dir(),
         ]
@@ -494,9 +502,9 @@ class scummvm(Runner):
 
     def get_run_data(self):
         env = self.get_env()
-        env["LD_LIBRARY_PATH"] = os.pathsep.join(filter(None, [
-            self.libs_dir,
-            env.get("LD_LIBRARY_PATH")]))
+        extra_libs = self.get_extra_libs()
+        if extra_libs:
+            env["LD_LIBRARY_PATH"] = ":".join([p for p in [extra_libs, env.get("LD_LIBRARY_PATH")] if p])
         return {"env": env, "command": self.get_command()}
 
     def inject_runner_option(self, command, key, cmdline, cmdline_empty=None):
@@ -518,12 +526,18 @@ class scummvm(Runner):
         for arg in split_arguments(args):
             command.append(arg)
         command.append(self.game_config.get("game_id"))
-        return {"command": command, "ld_library_path": self.libs_dir}
+        output = {"command": command}
+        extra_libs = self.get_extra_libs()
+        if extra_libs:
+            output["ld_library_path"] = extra_libs
+        return output
 
     def get_game_list(self):
         """Return the entire list of games supported by ScummVM."""
-        with subprocess.Popen([self.get_executable(), "--list-games"],
-                              stdout=subprocess.PIPE, encoding="utf-8", universal_newlines=True) as scummvm_process:
+        with subprocess.Popen(self.get_command() + ["--list-games"],
+                              stdout=subprocess.PIPE,
+                              encoding="utf-8",
+                              universal_newlines=True) as scummvm_process:
             scumm_output = scummvm_process.communicate()[0]
             game_list = str.split(scumm_output, "\n")
         game_array = []
